@@ -12,10 +12,9 @@ from lxml import etree
 
 from requests import session as requests_session
 
-from .base import Item, Storage
-from .http import prepare_auth, prepare_verify, USERAGENT
-
 from .. import exceptions, log, utils
+from .base import Item, Storage
+from .http import USERAGENT, prepare_auth, prepare_verify
 
 
 dav_logger = log.get(__name__)
@@ -226,6 +225,12 @@ class DavSession(object):
 class DavStorage(Storage):
 
     '''
+    .. note::
+
+        Please also see :doc:`server_support` for very important information,
+        as changing some of the default options might be very dangerous with
+        some servers.
+
     :param url: Base URL or an URL to a collection.
     :param username: Username for authentication.
     :param password: Password for authentication.
@@ -390,7 +395,7 @@ class DavStorage(Storage):
             headers=headers
         )
 
-    def _list(self, xml):
+    def _dav_query(self, xml):
         headers = self.session.get_default_headers()
 
         # CardDAV: The request MUST include a Depth header. The scope of the
@@ -439,12 +444,11 @@ class CaldavStorage(DavStorage):
     ''' + DavStorage.__doc__ + '''
     :param start_date: Start date of timerange to show, default -inf.
     :param end_date: End date of timerange to show, default +inf.
-    :param item_types: A tuple of collection types to show from the server.
-        For example, if you want to only get VEVENTs, pass ``('VEVENT',)``.
-        Falsy values mean "get all types". Dependent on server
-        functionality, no clientside validation of results. This currently
-        only affects the `list` method, but this shouldn't cause problems
-        in the normal usecase.
+    :param item_types: Comma-separated collection types to show from the
+        server. Dependent on server functionality, no clientside validation of
+        results. The empty value ``''`` is the same as ``'VTODO, VEVENT,
+        VJOURNAL'``.
+
     '''
 
     storage_name = 'caldav'
@@ -469,10 +473,11 @@ class CaldavStorage(DavStorage):
     get_multi_data_query = '{urn:ietf:params:xml:ns:caldav}calendar-data'
 
     def __init__(self, start_date=None, end_date=None,
-                 item_types=(), **kwargs):
+                 item_types='VTODO, VEVENT', **kwargs):
         super(CaldavStorage, self).__init__(**kwargs)
         if isinstance(item_types, str):
-            item_types = [x.strip() for x in item_types.split(',')]
+            item_types = filter(bool,
+                                (x.strip() for x in item_types.split(',')))
         self.item_types = tuple(item_types)
         if (start_date is None) != (end_date is None):
             raise ValueError('If start_date is given, '
@@ -489,29 +494,34 @@ class CaldavStorage(DavStorage):
     @staticmethod
     def _get_list_filters(components, start, end):
 
-        if not components:
-            components = ('VTODO', 'VEVENT')
-
-        caldavfilter = '''
-            <C:comp-filter name="VCALENDAR">
-                <C:comp-filter name="{component}">
-                    {timefilter}
+        if components:
+            caldavfilter = '''
+                <C:comp-filter name="VCALENDAR">
+                    <C:comp-filter name="{component}">
+                        {timefilter}
+                    </C:comp-filter>
                 </C:comp-filter>
-            </C:comp-filter>
-            '''
+                '''
 
-        if start is not None and end is not None:
-            start = start.strftime(CALDAV_DT_FORMAT)
-            end = end.strftime(CALDAV_DT_FORMAT)
+            if start is not None and end is not None:
+                start = start.strftime(CALDAV_DT_FORMAT)
+                end = end.strftime(CALDAV_DT_FORMAT)
 
-            timefilter = ('<C:time-range start="{start}" end="{end}"/>'
-                          .format(start=start, end=end))
+                timefilter = ('<C:time-range start="{start}" end="{end}"/>'
+                              .format(start=start, end=end))
+            else:
+                timefilter = ''
+
+            for component in components:
+                yield caldavfilter.format(component=component,
+                                          timefilter=timefilter)
         else:
-            timefilter = ''
-
-        for component in components:
-            yield caldavfilter.format(component=component,
-                                      timefilter=timefilter)
+            if start is not None and end is not None:
+                for x in CaldavStorage._get_list_filters(('VTODO', 'VEVENT'),
+                                                         start, end):
+                    yield x
+            else:
+                yield '<C:comp-filter name="VCALENDAR"/>'
 
     def list(self):
         data = '''<?xml version="1.0" encoding="utf-8" ?>
@@ -532,7 +542,7 @@ class CaldavStorage(DavStorage):
 
         for caldavfilter in caldavfilters:
             xml = data.format(caldavfilter=caldavfilter)
-            for href, etag in self._list(xml):
+            for href, etag in self._dav_query(xml):
                 assert href not in hrefs
                 hrefs.add(href)
                 yield href, etag
@@ -563,13 +573,44 @@ class CarddavStorage(DavStorage):
     get_multi_data_query = '{urn:ietf:params:xml:ns:carddav}address-data'
 
     def list(self):
-        return self._list('''<?xml version="1.0" encoding="utf-8" ?>
-            <C:addressbook-query xmlns:D="DAV:"
-                    xmlns:C="urn:ietf:params:xml:ns:carddav">
+        headers = self.session.get_default_headers()
+        headers['Depth'] = 1
+
+        data = '''<?xml version="1.0" encoding="utf-8" ?>
+            <D:propfind xmlns:D="DAV:">
                 <D:prop>
+                    <D:resourcetype/>
+                    <D:getcontenttype/>
                     <D:getetag/>
                 </D:prop>
-                <C:filter>
-                    <C:comp-filter name="VCARD"/>
-                </C:filter>
-            </C:addressbook-query>''')
+            </D:propfind>
+            '''
+
+        # We use a PROPFIND request instead of addressbook-query due to issues
+        # with Zimbra. See https://github.com/untitaker/vdirsyncer/issues/83
+        response = self.session.request('PROPFIND', '', data=data,
+                                        headers=headers)
+
+        root = etree.XML(response.content)
+        hrefs = set()
+        for element in root.iter('{DAV:}response'):
+            prop = element.find('{DAV:}propstat').find('{DAV:}prop')
+
+            resource_type = prop.find('{DAV:}resourcetype')
+            if resource_type.find('{DAV:}collection') is not None:
+                continue
+
+            content_type = prop.find('{DAV:}getcontenttype')
+            # different servers give different getcontenttypes:
+            # "text/vcard", "text/x-vcard", "text/x-vcard; charset=utf-8",
+            # "text/directory;profile=vCard", "text/directory",
+            # "text/vcard; charset=utf-8"
+            if 'vcard' not in content_type.text.lower():
+                continue
+
+            href = self._normalize_href(element.find('{DAV:}href').text)
+            etag = prop.find('{DAV:}getetag').text
+
+            if href not in hrefs:
+                hrefs.add(href)
+                yield self._normalize_href(href), etag
