@@ -8,10 +8,10 @@
 '''
 
 from .base import Item, Storage
-from ..utils import expand_path, get_password, request
-from ..utils.compat import text_type, urlparse
-from ..utils.vobject import split_collection
 from ..exceptions import NotFoundError
+from ..utils import expand_path, get_password, request
+from ..utils.compat import iteritems, text_type, urlparse
+from ..utils.vobject import split_collection
 
 USERAGENT = 'vdirsyncer'
 
@@ -49,12 +49,6 @@ class HttpStorage(Storage):
     Use a simple ``.ics`` file (or similar) from the web. Usable as ``http`` in
     the config file.
 
-    .. note::
-
-        This is a read-only storage. If you sync this with
-        read-and-write-storages (such as CalDAV), any changes on the other side
-        will get reverted.
-
     :param url: URL to the ``.ics`` file.
     :param username: Username for authentication.
     :param password: Password for authentication.
@@ -113,17 +107,14 @@ class HttpStorage(Storage):
     def list(self):
         r = request('GET', self.url, **self._settings)
         self._items = {}
-        rv = []
+
         for item in split_collection(r.text):
             item = Item(item)
             href = self._get_href(item)
             etag = item.hash
             self._items[href] = item, etag
-            rv.append((href, etag))
 
-        # we can't use yield here because we need to populate our
-        # dict even if the user doesn't exhaust the iterator
-        return rv
+        return ((href, etag) for href, (item, etag) in iteritems(self._items))
 
     def get(self, href):
         if self._items is None:
