@@ -8,10 +8,13 @@
 '''
 
 import os
+
+import click
+
 import requests
 
-from .. import log, exceptions
-from .compat import urlparse, get_raw_input
+from .. import exceptions, log
+from .compat import iteritems, urlparse
 
 
 logger = log.get(__name__)
@@ -35,13 +38,19 @@ def expand_path(p):
 
 def split_dict(d, f):
     '''Puts key into first dict if f(key), otherwise in second dict'''
-    a = {}
-    b = {}
-    for k, v in d.items():
-        if f(k):
-            a[k] = v
+    a, b = split_sequence(iteritems(d), lambda item: f(item[0]))
+    return dict(a), dict(b)
+
+
+def split_sequence(s, f):
+    '''Puts item into first list if f(item), else in second list'''
+    a = []
+    b = []
+    for item in s:
+        if f(item):
+            a.append(item)
         else:
-            b[k] = v
+            b.append(item)
 
     return a, b
 
@@ -109,7 +118,9 @@ def _password_from_keyring(username, resource):
 
         parsed = urlparse.urlsplit(key)
         path = parsed.path
-        if path.endswith('/'):
+        if not path:
+            return None
+        elif path.endswith('/'):
             path = path.rstrip('/')
         else:
             path = path.rsplit('/', 1)[0] + '/'
@@ -148,8 +159,6 @@ def get_password(username, resource):
 
 
     """
-    import getpass
-
     for func in (_password_from_netrc, _password_from_keyring):
         password = func(username, resource)
         if password is not None:
@@ -157,18 +166,14 @@ def get_password(username, resource):
                          .format(username, func.__doc__))
             return password
 
-    prompt = ('Server password for {} at the resource {}: '
+    prompt = ('Server password for {} at the resource {}'
               .format(username, resource))
-    password = getpass.getpass(prompt=prompt)
+    password = click.prompt(prompt, hide_input=True)
 
-    if keyring is not None:
-        answer = None
-        while answer not in ['', 'y', 'n']:
-            prompt = 'Save this password in the keyring? [y/N] '
-            answer = get_raw_input(prompt).lower()
-        if answer == 'y':
-            keyring.set_password(password_key_prefix + resource,
-                                 username, password)
+    if keyring is not None and \
+       click.confirm('Save this password in the keyring?', default=False):
+        keyring.set_password(password_key_prefix + resource,
+                             username, password)
 
     return password
 
