@@ -9,11 +9,11 @@
 import hashlib
 
 import icalendar.cal
-import icalendar.parser
 import icalendar.caselessdict
+import icalendar.parser
 
-from . import cached_property
-from .compat import text_type, itervalues
+from . import cached_property, split_sequence
+from .compat import itervalues, text_type
 
 
 def _process_properties(*s):
@@ -50,18 +50,19 @@ ICALENDAR_ORIGINAL_ORDER_SUPPORT = \
 
 class Item(object):
 
-    '''should-be-immutable wrapper class for VCALENDAR (VEVENT, VTODO) and
+    '''Immutable wrapper class for VCALENDAR (VEVENT, VTODO) and
     VCARD'''
 
     def __init__(self, raw):
         assert isinstance(raw, text_type)
-
         self._raw = raw
 
     @cached_property
     def raw(self):
-        '''Raw content of the item, which vdirsyncer doesn't validate in any
-        way.'''
+        '''Raw content of the item, as unicode string.
+
+        Vdirsyncer doesn't validate the content in any way.
+        '''
         return self._raw
 
     @cached_property
@@ -128,25 +129,33 @@ def split_collection(text, inline=(u'VTIMEZONE',),
                      wrap_items_with=(u'VCALENDAR',)):
     '''Emits items in the order they occur in the text.'''
     assert isinstance(text, text_type)
-    collection = icalendar.cal.Component.from_ical(text)
-    items = collection.subcomponents
+    collections = icalendar.cal.Component.from_ical(text, multiple=True)
+    assert collections
+    collection_name = None
 
-    if collection.name in wrap_items_with:
-        start = u'BEGIN:{}'.format(collection.name)
-        end = u'END:{}'.format(collection.name)
-    else:
-        start = end = u''
+    for collection in collections:
+        items = collection.subcomponents
+        if collection_name is None:
+            collection_name = collection.name
 
-    inlined_items = {}
-    for item in items:
-        if item.name in inline:
-            inlined_items[item.name] = item
+            if collection_name in wrap_items_with:
+                start = u'BEGIN:{}'.format(collection_name)
+                end = u'END:{}'.format(collection_name)
+            else:
+                start = end = u''
 
-    for item in items:
-        if item.name not in inline:
+        if collection.name != collection_name:
+            raise ValueError('Different types of components at top-level. '
+                             'Expected {}, got {}.'
+                             .format(collection_name, collection.name))
+
+        inlined_items, normal_items = \
+            split_sequence(items, lambda item: item.name in inline)
+
+        for item in normal_items:
             lines = []
             lines.append(start)
-            for inlined_item in itervalues(inlined_items):
+            for inlined_item in inlined_items:
                 lines.extend(to_unicode_lines(inlined_item))
 
             lines.extend(to_unicode_lines(item))
@@ -173,15 +182,21 @@ def to_unicode_lines(item):
             yield icalendar.parser.foldline(content_line)
 
 
-def join_collection(items, wrappers={
+_default_join_wrappers = {
     u'VCALENDAR': (u'VCALENDAR', (u'VTIMEZONE',)),
     u'VCARD': (u'VADDRESSBOOK', ())
-}):
+}
+
+
+def join_collection(items, wrappers=None):
     '''
     :param wrappers: {
         item_type: wrapper_type, items_to_inline
     }
     '''
+    if wrappers is None:
+        wrappers = _default_join_wrappers
+
     inline = {}
     components = []
     wrapper_type = None
